
#ifndef VTKINFOVISBOOSTGRAPHALGORITHMS_EXPORT_H
#define VTKINFOVISBOOSTGRAPHALGORITHMS_EXPORT_H

#ifdef VTKINFOVISBOOSTGRAPHALGORITHMS_STATIC_DEFINE
#  define VTKINFOVISBOOSTGRAPHALGORITHMS_EXPORT
#  define VTKINFOVISBOOSTGRAPHALGORITHMS_NO_EXPORT
#else
#  ifndef VTKINFOVISBOOSTGRAPHALGORITHMS_EXPORT
#    ifdef InfovisBoostGraphAlgorithms_EXPORTS
        /* We are building this library */
#      define VTKINFOVISBOOSTGRAPHALGORITHMS_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define VTKINFOVISBOOSTGRAPHALGORITHMS_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef VTKINFOVISBOOSTGRAPHALGORITHMS_NO_EXPORT
#    define VTKINFOVISBOOSTGRAPHALGORITHMS_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef VTKINFOVISBOOSTGRAPHALGORITHMS_DEPRECATED
#  define VTKINFOVISBOOSTGRAPHALGORITHMS_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef VTKINFOVISBOOSTGRAPHALGORITHMS_DEPRECATED_EXPORT
#  define VTKINFOVISBOOSTGRAPHALGORITHMS_DEPRECATED_EXPORT VTKINFOVISBOOSTGRAPHALGORITHMS_EXPORT VTKINFOVISBOOSTGRAPHALGORITHMS_DEPRECATED
#endif

#ifndef VTKINFOVISBOOSTGRAPHALGORITHMS_DEPRECATED_NO_EXPORT
#  define VTKINFOVISBOOSTGRAPHALGORITHMS_DEPRECATED_NO_EXPORT VTKINFOVISBOOSTGRAPHALGORITHMS_NO_EXPORT VTKINFOVISBOOSTGRAPHALGORITHMS_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKINFOVISBOOSTGRAPHALGORITHMS_NO_DEPRECATED
#    define VTKINFOVISBOOSTGRAPHALGORITHMS_NO_DEPRECATED
#  endif
#endif

/* VTK-HeaderTest-Exclude: vtkInfovisBoostGraphAlgorithmsModule.h */

/* Include ABI Namespace */
#include "vtkABINamespace.h"

#endif /* VTKINFOVISBOOSTGRAPHALGORITHMS_EXPORT_H */
