#ifndef vtk_glx_mangle_h
#define vtk_glx_mangle_h

#define GLAD_GLX_VERSION_1_0 VTK_GLAD_GLX_VERSION_1_0
#define GLAD_GLX_VERSION_1_1 VTK_GLAD_GLX_VERSION_1_1
#define GLAD_GLX_VERSION_1_2 VTK_GLAD_GLX_VERSION_1_2
#define GLAD_GLX_VERSION_1_3 VTK_GLAD_GLX_VERSION_1_3
#define GLAD_GLX_VERSION_1_4 VTK_GLAD_GLX_VERSION_1_4
#define GLAD_GLX_3DFX_multisample VTK_GLAD_GLX_3DFX_multisample
#define GLAD_GLX_AMD_gpu_association VTK_GLAD_GLX_AMD_gpu_association
#define GLAD_GLX_ARB_context_flush_control VTK_GLAD_GLX_ARB_context_flush_control
#define GLAD_GLX_ARB_create_context VTK_GLAD_GLX_ARB_create_context
#define GLAD_GLX_ARB_create_context_no_error VTK_GLAD_GLX_ARB_create_context_no_error
#define GLAD_GLX_ARB_create_context_profile VTK_GLAD_GLX_ARB_create_context_profile
#define GLAD_GLX_ARB_create_context_robustness VTK_GLAD_GLX_ARB_create_context_robustness
#define GLAD_GLX_ARB_fbconfig_float VTK_GLAD_GLX_ARB_fbconfig_float
#define GLAD_GLX_ARB_framebuffer_sRGB VTK_GLAD_GLX_ARB_framebuffer_sRGB
#define GLAD_GLX_ARB_get_proc_address VTK_GLAD_GLX_ARB_get_proc_address
#define GLAD_GLX_ARB_multisample VTK_GLAD_GLX_ARB_multisample
#define GLAD_GLX_ARB_robustness_application_isolation                                              \
  VTK_GLAD_GLX_ARB_robustness_application_isolation
#define GLAD_GLX_ARB_robustness_share_group_isolation                                              \
  VTK_GLAD_GLX_ARB_robustness_share_group_isolation
#define GLAD_GLX_ARB_vertex_buffer_object VTK_GLAD_GLX_ARB_vertex_buffer_object
#define GLAD_GLX_EXT_buffer_age VTK_GLAD_GLX_EXT_buffer_age
#define GLAD_GLX_EXT_context_priority VTK_GLAD_GLX_EXT_context_priority
#define GLAD_GLX_EXT_create_context_es2_profile VTK_GLAD_GLX_EXT_create_context_es2_profile
#define GLAD_GLX_EXT_create_context_es_profile VTK_GLAD_GLX_EXT_create_context_es_profile
#define GLAD_GLX_EXT_fbconfig_packed_float VTK_GLAD_GLX_EXT_fbconfig_packed_float
#define GLAD_GLX_EXT_framebuffer_sRGB VTK_GLAD_GLX_EXT_framebuffer_sRGB
#define GLAD_GLX_EXT_get_drawable_type VTK_GLAD_GLX_EXT_get_drawable_type
#define GLAD_GLX_EXT_import_context VTK_GLAD_GLX_EXT_import_context
#define GLAD_GLX_EXT_libglvnd VTK_GLAD_GLX_EXT_libglvnd
#define GLAD_GLX_EXT_no_config_context VTK_GLAD_GLX_EXT_no_config_context
#define GLAD_GLX_EXT_stereo_tree VTK_GLAD_GLX_EXT_stereo_tree
#define GLAD_GLX_EXT_swap_control VTK_GLAD_GLX_EXT_swap_control
#define GLAD_GLX_EXT_swap_control_tear VTK_GLAD_GLX_EXT_swap_control_tear
#define GLAD_GLX_EXT_texture_from_pixmap VTK_GLAD_GLX_EXT_texture_from_pixmap
#define GLAD_GLX_EXT_visual_info VTK_GLAD_GLX_EXT_visual_info
#define GLAD_GLX_EXT_visual_rating VTK_GLAD_GLX_EXT_visual_rating
#define GLAD_GLX_INTEL_swap_event VTK_GLAD_GLX_INTEL_swap_event
#define GLAD_GLX_MESA_agp_offset VTK_GLAD_GLX_MESA_agp_offset
#define GLAD_GLX_MESA_copy_sub_buffer VTK_GLAD_GLX_MESA_copy_sub_buffer
#define GLAD_GLX_MESA_pixmap_colormap VTK_GLAD_GLX_MESA_pixmap_colormap
#define GLAD_GLX_MESA_query_renderer VTK_GLAD_GLX_MESA_query_renderer
#define GLAD_GLX_MESA_release_buffers VTK_GLAD_GLX_MESA_release_buffers
#define GLAD_GLX_MESA_set_3dfx_mode VTK_GLAD_GLX_MESA_set_3dfx_mode
#define GLAD_GLX_MESA_swap_control VTK_GLAD_GLX_MESA_swap_control
#define GLAD_GLX_NV_copy_buffer VTK_GLAD_GLX_NV_copy_buffer
#define GLAD_GLX_NV_copy_image VTK_GLAD_GLX_NV_copy_image
#define GLAD_GLX_NV_delay_before_swap VTK_GLAD_GLX_NV_delay_before_swap
#define GLAD_GLX_NV_float_buffer VTK_GLAD_GLX_NV_float_buffer
#define GLAD_GLX_NV_multigpu_context VTK_GLAD_GLX_NV_multigpu_context
#define GLAD_GLX_NV_multisample_coverage VTK_GLAD_GLX_NV_multisample_coverage
#define GLAD_GLX_NV_present_video VTK_GLAD_GLX_NV_present_video
#define GLAD_GLX_NV_robustness_video_memory_purge VTK_GLAD_GLX_NV_robustness_video_memory_purge
#define GLAD_GLX_NV_swap_group VTK_GLAD_GLX_NV_swap_group
#define GLAD_GLX_NV_video_capture VTK_GLAD_GLX_NV_video_capture
#define GLAD_GLX_NV_video_out VTK_GLAD_GLX_NV_video_out
#define GLAD_GLX_OML_swap_method VTK_GLAD_GLX_OML_swap_method
#define GLAD_GLX_OML_sync_control VTK_GLAD_GLX_OML_sync_control
#define GLAD_GLX_SGIS_blended_overlay VTK_GLAD_GLX_SGIS_blended_overlay
#define GLAD_GLX_SGIS_multisample VTK_GLAD_GLX_SGIS_multisample
#define GLAD_GLX_SGIS_shared_multisample VTK_GLAD_GLX_SGIS_shared_multisample
#define GLAD_GLX_SGIX_fbconfig VTK_GLAD_GLX_SGIX_fbconfig
#define GLAD_GLX_SGIX_hyperpipe VTK_GLAD_GLX_SGIX_hyperpipe
#define GLAD_GLX_SGIX_pbuffer VTK_GLAD_GLX_SGIX_pbuffer
#define GLAD_GLX_SGIX_swap_barrier VTK_GLAD_GLX_SGIX_swap_barrier
#define GLAD_GLX_SGIX_swap_group VTK_GLAD_GLX_SGIX_swap_group
#define GLAD_GLX_SGIX_video_resize VTK_GLAD_GLX_SGIX_video_resize
#define GLAD_GLX_SGIX_visual_select_group VTK_GLAD_GLX_SGIX_visual_select_group
#define GLAD_GLX_SGI_cushion VTK_GLAD_GLX_SGI_cushion
#define GLAD_GLX_SGI_make_current_read VTK_GLAD_GLX_SGI_make_current_read
#define GLAD_GLX_SGI_swap_control VTK_GLAD_GLX_SGI_swap_control
#define GLAD_GLX_SGI_video_sync VTK_GLAD_GLX_SGI_video_sync
#define GLAD_GLX_SUN_get_transparent_index VTK_GLAD_GLX_SUN_get_transparent_index

#define glad_glXBindChannelToWindowSGIX vtk_glad_glXBindChannelToWindowSGIX
#define glad_glXBindHyperpipeSGIX vtk_glad_glXBindHyperpipeSGIX
#define glad_glXBindSwapBarrierNV vtk_glad_glXBindSwapBarrierNV
#define glad_glXBindSwapBarrierSGIX vtk_glad_glXBindSwapBarrierSGIX
#define glad_glXBindTexImageEXT vtk_glad_glXBindTexImageEXT
#define glad_glXBindVideoCaptureDeviceNV vtk_glad_glXBindVideoCaptureDeviceNV
#define glad_glXBindVideoDeviceNV vtk_glad_glXBindVideoDeviceNV
#define glad_glXBindVideoImageNV vtk_glad_glXBindVideoImageNV
#define glad_glXBlitContextFramebufferAMD vtk_glad_glXBlitContextFramebufferAMD
#define glad_glXChannelRectSGIX vtk_glad_glXChannelRectSGIX
#define glad_glXChannelRectSyncSGIX vtk_glad_glXChannelRectSyncSGIX
#define glad_glXChooseFBConfig vtk_glad_glXChooseFBConfig
#define glad_glXChooseFBConfigSGIX vtk_glad_glXChooseFBConfigSGIX
#define glad_glXChooseVisual vtk_glad_glXChooseVisual
#define glad_glXCopyBufferSubDataNV vtk_glad_glXCopyBufferSubDataNV
#define glad_glXCopyContext vtk_glad_glXCopyContext
#define glad_glXCopyImageSubDataNV vtk_glad_glXCopyImageSubDataNV
#define glad_glXCopySubBufferMESA vtk_glad_glXCopySubBufferMESA
#define glad_glXCreateAssociatedContextAMD vtk_glad_glXCreateAssociatedContextAMD
#define glad_glXCreateAssociatedContextAttribsAMD vtk_glad_glXCreateAssociatedContextAttribsAMD
#define glad_glXCreateContext vtk_glad_glXCreateContext
#define glad_glXCreateContextAttribsARB vtk_glad_glXCreateContextAttribsARB
#define glad_glXCreateContextWithConfigSGIX vtk_glad_glXCreateContextWithConfigSGIX
#define glad_glXCreateGLXPbufferSGIX vtk_glad_glXCreateGLXPbufferSGIX
#define glad_glXCreateGLXPixmap vtk_glad_glXCreateGLXPixmap
#define glad_glXCreateGLXPixmapMESA vtk_glad_glXCreateGLXPixmapMESA
#define glad_glXCreateGLXPixmapWithConfigSGIX vtk_glad_glXCreateGLXPixmapWithConfigSGIX
#define glad_glXCreateNewContext vtk_glad_glXCreateNewContext
#define glad_glXCreatePbuffer vtk_glad_glXCreatePbuffer
#define glad_glXCreatePixmap vtk_glad_glXCreatePixmap
#define glad_glXCreateWindow vtk_glad_glXCreateWindow
#define glad_glXCushionSGI vtk_glad_glXCushionSGI
#define glad_glXDelayBeforeSwapNV vtk_glad_glXDelayBeforeSwapNV
#define glad_glXDeleteAssociatedContextAMD vtk_glad_glXDeleteAssociatedContextAMD
#define glad_glXDestroyContext vtk_glad_glXDestroyContext
#define glad_glXDestroyGLXPbufferSGIX vtk_glad_glXDestroyGLXPbufferSGIX
#define glad_glXDestroyGLXPixmap vtk_glad_glXDestroyGLXPixmap
#define glad_glXDestroyHyperpipeConfigSGIX vtk_glad_glXDestroyHyperpipeConfigSGIX
#define glad_glXDestroyPbuffer vtk_glad_glXDestroyPbuffer
#define glad_glXDestroyPixmap vtk_glad_glXDestroyPixmap
#define glad_glXDestroyWindow vtk_glad_glXDestroyWindow
#define glad_glXEnumerateVideoCaptureDevicesNV vtk_glad_glXEnumerateVideoCaptureDevicesNV
#define glad_glXEnumerateVideoDevicesNV vtk_glad_glXEnumerateVideoDevicesNV
#define glad_glXFreeContextEXT vtk_glad_glXFreeContextEXT
#define glad_glXGetAGPOffsetMESA vtk_glad_glXGetAGPOffsetMESA
#define glad_glXGetClientString vtk_glad_glXGetClientString
#define glad_glXGetConfig vtk_glad_glXGetConfig
#define glad_glXGetContextGPUIDAMD vtk_glad_glXGetContextGPUIDAMD
#define glad_glXGetContextIDEXT vtk_glad_glXGetContextIDEXT
#define glad_glXGetCurrentAssociatedContextAMD vtk_glad_glXGetCurrentAssociatedContextAMD
#define glad_glXGetCurrentContext vtk_glad_glXGetCurrentContext
#define glad_glXGetCurrentDisplay vtk_glad_glXGetCurrentDisplay
#define glad_glXGetCurrentDisplayEXT vtk_glad_glXGetCurrentDisplayEXT
#define glad_glXGetCurrentDrawable vtk_glad_glXGetCurrentDrawable
#define glad_glXGetCurrentReadDrawable vtk_glad_glXGetCurrentReadDrawable
#define glad_glXGetCurrentReadDrawableSGI vtk_glad_glXGetCurrentReadDrawableSGI
#define glad_glXGetFBConfigAttrib vtk_glad_glXGetFBConfigAttrib
#define glad_glXGetFBConfigAttribSGIX vtk_glad_glXGetFBConfigAttribSGIX
#define glad_glXGetFBConfigFromVisualSGIX vtk_glad_glXGetFBConfigFromVisualSGIX
#define glad_glXGetFBConfigs vtk_glad_glXGetFBConfigs
#define glad_glXGetGPUIDsAMD vtk_glad_glXGetGPUIDsAMD
#define glad_glXGetGPUInfoAMD vtk_glad_glXGetGPUInfoAMD
#define glad_glXGetMscRateOML vtk_glad_glXGetMscRateOML
#define glad_glXGetProcAddress vtk_glad_glXGetProcAddress
#define glad_glXGetProcAddressARB vtk_glad_glXGetProcAddressARB
#define glad_glXGetSelectedEvent vtk_glad_glXGetSelectedEvent
#define glad_glXGetSelectedEventSGIX vtk_glad_glXGetSelectedEventSGIX
#define glad_glXGetSwapIntervalMESA vtk_glad_glXGetSwapIntervalMESA
#define glad_glXGetSyncValuesOML vtk_glad_glXGetSyncValuesOML
#define glad_glXGetTransparentIndexSUN vtk_glad_glXGetTransparentIndexSUN
#define glad_glXGetVideoDeviceNV vtk_glad_glXGetVideoDeviceNV
#define glad_glXGetVideoInfoNV vtk_glad_glXGetVideoInfoNV
#define glad_glXGetVideoSyncSGI vtk_glad_glXGetVideoSyncSGI
#define glad_glXGetVisualFromFBConfig vtk_glad_glXGetVisualFromFBConfig
#define glad_glXGetVisualFromFBConfigSGIX vtk_glad_glXGetVisualFromFBConfigSGIX
#define glad_glXHyperpipeAttribSGIX vtk_glad_glXHyperpipeAttribSGIX
#define glad_glXHyperpipeConfigSGIX vtk_glad_glXHyperpipeConfigSGIX
#define glad_glXImportContextEXT vtk_glad_glXImportContextEXT
#define glad_glXIsDirect vtk_glad_glXIsDirect
#define glad_glXJoinSwapGroupNV vtk_glad_glXJoinSwapGroupNV
#define glad_glXJoinSwapGroupSGIX vtk_glad_glXJoinSwapGroupSGIX
#define glad_glXLockVideoCaptureDeviceNV vtk_glad_glXLockVideoCaptureDeviceNV
#define glad_glXMakeAssociatedContextCurrentAMD vtk_glad_glXMakeAssociatedContextCurrentAMD
#define glad_glXMakeContextCurrent vtk_glad_glXMakeContextCurrent
#define glad_glXMakeCurrent vtk_glad_glXMakeCurrent
#define glad_glXMakeCurrentReadSGI vtk_glad_glXMakeCurrentReadSGI
#define glad_glXNamedCopyBufferSubDataNV vtk_glad_glXNamedCopyBufferSubDataNV
#define glad_glXQueryChannelDeltasSGIX vtk_glad_glXQueryChannelDeltasSGIX
#define glad_glXQueryChannelRectSGIX vtk_glad_glXQueryChannelRectSGIX
#define glad_glXQueryContext vtk_glad_glXQueryContext
#define glad_glXQueryContextInfoEXT vtk_glad_glXQueryContextInfoEXT
#define glad_glXQueryCurrentRendererIntegerMESA vtk_glad_glXQueryCurrentRendererIntegerMESA
#define glad_glXQueryCurrentRendererStringMESA vtk_glad_glXQueryCurrentRendererStringMESA
#define glad_glXQueryDrawable vtk_glad_glXQueryDrawable
#define glad_glXQueryExtension vtk_glad_glXQueryExtension
#define glad_glXQueryExtensionsString vtk_glad_glXQueryExtensionsString
#define glad_glXQueryFrameCountNV vtk_glad_glXQueryFrameCountNV
#define glad_glXQueryGLXPbufferSGIX vtk_glad_glXQueryGLXPbufferSGIX
#define glad_glXQueryHyperpipeAttribSGIX vtk_glad_glXQueryHyperpipeAttribSGIX
#define glad_glXQueryHyperpipeBestAttribSGIX vtk_glad_glXQueryHyperpipeBestAttribSGIX
#define glad_glXQueryHyperpipeConfigSGIX vtk_glad_glXQueryHyperpipeConfigSGIX
#define glad_glXQueryHyperpipeNetworkSGIX vtk_glad_glXQueryHyperpipeNetworkSGIX
#define glad_glXQueryMaxSwapBarriersSGIX vtk_glad_glXQueryMaxSwapBarriersSGIX
#define glad_glXQueryMaxSwapGroupsNV vtk_glad_glXQueryMaxSwapGroupsNV
#define glad_glXQueryRendererIntegerMESA vtk_glad_glXQueryRendererIntegerMESA
#define glad_glXQueryRendererStringMESA vtk_glad_glXQueryRendererStringMESA
#define glad_glXQueryServerString vtk_glad_glXQueryServerString
#define glad_glXQuerySwapGroupNV vtk_glad_glXQuerySwapGroupNV
#define glad_glXQueryVersion vtk_glad_glXQueryVersion
#define glad_glXQueryVideoCaptureDeviceNV vtk_glad_glXQueryVideoCaptureDeviceNV
#define glad_glXReleaseBuffersMESA vtk_glad_glXReleaseBuffersMESA
#define glad_glXReleaseTexImageEXT vtk_glad_glXReleaseTexImageEXT
#define glad_glXReleaseVideoCaptureDeviceNV vtk_glad_glXReleaseVideoCaptureDeviceNV
#define glad_glXReleaseVideoDeviceNV vtk_glad_glXReleaseVideoDeviceNV
#define glad_glXReleaseVideoImageNV vtk_glad_glXReleaseVideoImageNV
#define glad_glXResetFrameCountNV vtk_glad_glXResetFrameCountNV
#define glad_glXSelectEvent vtk_glad_glXSelectEvent
#define glad_glXSelectEventSGIX vtk_glad_glXSelectEventSGIX
#define glad_glXSendPbufferToVideoNV vtk_glad_glXSendPbufferToVideoNV
#define glad_glXSet3DfxModeMESA vtk_glad_glXSet3DfxModeMESA
#define glad_glXSwapBuffers vtk_glad_glXSwapBuffers
#define glad_glXSwapBuffersMscOML vtk_glad_glXSwapBuffersMscOML
#define glad_glXSwapIntervalEXT vtk_glad_glXSwapIntervalEXT
#define glad_glXSwapIntervalMESA vtk_glad_glXSwapIntervalMESA
#define glad_glXSwapIntervalSGI vtk_glad_glXSwapIntervalSGI
#define glad_glXUseXFont vtk_glad_glXUseXFont
#define glad_glXWaitForMscOML vtk_glad_glXWaitForMscOML
#define glad_glXWaitForSbcOML vtk_glad_glXWaitForSbcOML
#define glad_glXWaitGL vtk_glad_glXWaitGL
#define glad_glXWaitVideoSyncSGI vtk_glad_glXWaitVideoSyncSGI
#define glad_glXWaitX vtk_glad_glXWaitX

#define gladLoadGLXUserPtr vtk_gladLoadGLXUserPtr
#define gladLoadGLX vtk_gladLoadGLX
#define gladLoaderLoadGLX vtk_gladLoaderLoadGLX
#define gladLoaderUnloadGLX vtk_gladLoaderUnloadGLX

#endif
